// List.h - Written by Walter O. Krawec for Small C++
// a BETA version of a linked list - fully working for the most part however
// has not been fully tested or implemented yet.
// Keep checking the Small C++ website for updates to this file
//
// USAGE:
//
// include this list.h file in your program specifying whatever types you need to use.  For example
// if you need two different types of lists - one for bytes and one for int16s - then do the following:
//
// #include <ADT\List.h> <byte>
// #include <ADT\List.h> <int16>
//
// also include Memory.h and create an instance of a Heap object (called 'heap' in these examples)
// define an instance of the list like so:
//
// adt::list<TYPE>* listName;
// listName = adt::list<TYPE>.new(&heap)
//
// where TYPE is one of the #include'd types (byte or int16 in this example) - see the manual for more
// info on using templates in Small C++
//
// Now, to add to a list: listName.add(DATA);
// will add DATA to the front of the list
// To cycle through the list, use the iterator class:
//
// adt::list<TYPE>::iterator Iter;
// Iter.set(listName.begin());
// while(!Iter.endOfList) {
//   DATA = Iter.get();
//   Iter.next();
// }
//
// To remove a node, use the iterator to find the node you want to remove, then call the remove function like so:
//
// listName.remove(Iter.getNode());
//

#ifndef ADT_LIST_H_<>
#define ADT_LIST_H_<>

#include <Memory.h>

#define T TEMPLATE_TYPE 0

namespace adt
{
	template class list
	{
		struct node
		{
			adt::list<>::node* next, *prev;
			T data;
		}

	private:
		adt::list<>::node* head;
		memory::heap* heap;

	public:
		list()
		{
			head = 0;
			heap = 0;
		}

		static adt::list<>* new(memory::heap* heap)
		{
			adt::list<>* p;
			p = (adt::list<>*)heap.malloc(adt::list<>.size);
			p._00();
			p.heap = heap;
			return (p);
		}

		void add(T data)
		{
			adt::list<>::node* save;
			save = head;
			head = (adt::list<>::node*)heap.malloc(adt::list<>::node.size);
			if(save != 0) {
				save.prev = head;
			}
			head.next = save;
			head.prev = 0;
			head.data = data;
		}

		void remove(adt::list<>::node* node)
		{
			if(node == 0) {return ();}
			if(node == head)
			{
				head = head.next;
				head.prev = 0;

				heap.free((void*)node);
			}
			else
			{
				if(node.prev != 0){
					node.prev.next = node.next;
				}
				if(node.next != 0){
					node.next.prev = node.prev;
				}

				heap.free((void*)node);
			}
		}

		adt::list<>::node* begin()
		{
			return (head);
		}

		class iterator
		{
		private:
			adt::list<>::node* currentNode;
		public:
			void iterator()
			{
				currentNode = 0;
			}
			void set(adt::list<>::node* node)
			{
				currentNode = node;
			}
			void next()
			{
				if(currentNode != 0)
				{
					currentNode = currentNode.next;
				}
			}
			T get()
			{
				if(currentNode == 0)
				{
					return (0);
				}

				return (currentNode.data);
			}
			adt::list<>::node* getNode()
			{
				return (currentNode);
			}

			byte endOfList()
			{
				if(currentNode == 0)
				{
					return (1);
				}

				return (0);
			}
		}
	}
}

#endif