// SCI.cpp - by Walter O. Krawec
// basic code for using the HC11's SCI port.
//
// USAGE:
//
// Create an instance of the class:
// SCI name;
//
// that will automatically setup the SCI port.  Now to output a character:
//
// name.out(BYTE_TO_OUTPUT);
//
// or input:
//
// name.in(&c);
//
// 'c' will then store the sent data (if any).  'in' returns a 1 if there was any data sent, 0 otherwise.
//

#ifndef SCI_H_
#define SCI_H_

#define char byte

namespace io
{
	class SCI
	{
	public:
		static void setup()
		{
			__AOC {
				ASM: ldaa #$30
					 staa $102b
					 ldaa #$0c
					 staa $102d;
			}
		}

		static void out(char c)
		{
			__AOC {
				LABEL lblSCIOUT;
				ASM: ldaa $102e
					 ora  #$7f
					 cmpa #$ff
					 bne  lblSCIOUT;
				GET c;
				ASM: ldaa 0,x
					 staa $102f;
			}
		}

		static byte in(char* output)
		{
			__AOC {
				ASM: ldaa $102e
					 ora #$df
					 cmpa #$ff
					 bne lblSCIINRTS;
				GET output;
				INDIRECT;
				ASM: ldaa $102f
					 staa 0,x;
			}
			return (1);		// everything went OK

			__AOC {
				LABEL lblSCIINRTS;
			}
			return (0);		// nothing in the buffer
		}
	}
}

#endif