// Memory.cpp - Written by Walter O. Krawec for Small C++
// implementation of a dynamic memory system for user programs
// see memory.h for usage instructions
//
#include <Memory.h>

namespace memory
{
	class heap
	{
		void setup(void* address, void* size)
		{
			startAddress = address;
			__AOC {
				GET address;
				ASM: ldx 0,x
					 ldaa #$01
					 staa 0,x
					 ldd  #$0000
					 std  1,x
					 pshx;
				STACK_INC 2;
				GET size;
				ASM: ldd 0,x
					 pulx
					 std 3,x;
				STACK_DEC 2;
			}
		}

		void* malloc(void* block_size)
		{
			void* address;
			address = startAddress;

			__AOC {
				GET block_size;
				ASM: ldd 0,x				* save the block_size in Y
					 addd #$05				* node size = 5 bytes, so take this into account...
					 xgdy;
				GET address;
				ASM: ldd 0,x
					 psha
					 pshb
					 ldd #$0000
					 std 0,x
					 pulb
					 pula
					 xgdx;
				LABEL HEAP_MALLOC_0;
				ASM: cmpx #$0000			* is this node NULL?
					 beq HEAP_MALLOC_DONE	* yes, so quit

					 ldaa 0,x				* is this block free?
					 cmpa #$01
					 bne HEAP_MALLOC_1

					 cmpy  3,x				* OK, is it big enough?
					 bgt  HEAP_MALLOC_1

					 ldaa #$00				* this node is no longer free
					 staa 0,x
					 ldd  3,x				* save this node's block size...
					 pshb					* ...on the stack
					 psha

					 xgdy					* restore the size of this block
					 subd #$05				* remove the node size
					 std  3,x				* and store this size in the current node
					 addd #$05
					 xgdy					* save the size again in Y

					 ldd 1,x				* save the node 'next' pointer on the stack
					 psha
					 pshb

					 pshy					* push the size onto the stack
					 tsy					* the following will add the current addres
					 pshx					* to the node block size giving us an
					 xgdx
					 addd 0,y				* address to the next node
					 pulx					* so, X = node address (still), D = calculation result
					 puly					* and Y is once again block_size+sizeof(node)

					 std 1,x				* this node's next value is now the above calculation result

					 pulb					* restore the old 'next' pointer from the stack
					 pula

					 pshx					* save the current node address
					 ldx  1,x				* get the 'next' node's address
					 std  1,x				* and save the saved next address to this node's next address
					 ldaa #$01
					 staa 0,x				* this new node is now free

					 pshy					* push the block_size+sizeof(node) value to the stack
					 tsy					* point to the stack
					 ldd  4,y				* load the old block size (stored on the stack 4 bytes above)
					 subd 0,y				* and subtract it from the old block size
					 std 3,x				* this is now the size of this new node

					 puly
					 pulx					* restore the current node address
					 xgdx
					 addd #$05				* add 5 to this address
					 puly					* cleanup

					 pshb					* save this address to the stack
					 psha;
				STACK_INC 2;
				GET address;
				ASM: pula
					 pulb
					 std 0,x
					 jmp HEAP_MALLOC_DONE;
				STACK_DEC 2;
				LABEL HEAP_MALLOC_1;
				ASM: ldx 1,x
					 jmp HEAP_MALLOC_0;
				LABEL HEAP_MALLOC_DONE;
			}

			return (address);
		}

		void free(void* p)
		{
			__AOC {
				GET p;
				ASM: ldd 0,x
					 subd #$05
					 xgdx
					 ldaa #$01
					 staa 0,x;
			}
		}
	}
}