// Memory.h - Written by Walter O. Krawec for Small C++
// implementation of a dynamic memory system for user programs
//
// USAGE:
//
// first, create an instance of the heap class (recommend one per program - just pass POINTERS to it where needed):
// memory::heap heap;
//
// in this example I call it 'heap' but you can change that name of course.  Now, setup the heap object by giving
// it the location of the RAM that it can use (remember, the heap builds upwards unlike the stack, so pass the lowest
// address in RAM you can):
//
// heap.setup(0x7fff, 0x200);
//
// in this example, the heap will exist at location 0x7fff and contain 512 bytes max (the second argument).
// Now that it's setup, you can begin to use it.  To create a new instance of a class (in this example called
// CLASS_TYPE):
//
// CLASS_TYPE* name;
// name = heap.malloc(CLASS_TYPE.size);
//
// name will now point to a location in the heap where the new object was allocated (0x0000 if not enough memory).
// to free the above pointer (do this when you no longer need it):
//
// heap.free(name);
//
// NOTE: Don't malloc and free too frequently - only when you need to in order to prevent memory fragmentation.
//

#ifndef MEMORY_HEAP_H_
#define MEMORY_HEAP_H_

namespace memory
{
	class heap
	{
	public:
		void* malloc(void* block_size);
		void free(void* p);
		void setup(void* address, void* size);

		heap()
		{
			setup(0x0000, 0x0000);
		}

	private:
		void* startAddress;
	}
}

#endif