// Test3.cpp is a program that demonstrates the use of the linked lists and dynamic memory.  See Memory.h and List.h for more info
// to use this program, run a terminal editor on your PC
//

#include <SCI.h>
#include <StandardIO.h>
#include <Memory.cpp>			// allows us to use the dynamic memory object

#include <ADT\List.h> <int16>	// a list for int16s
#include <ADT\List.h> <byte>	// a list for bytes

#define HEAP_MEMORY_START_ADDRESS 0x2000
#define HEAP_MEMORY_SIZE 0x100

class TestProgram3
{
	// printList16 will print out the contents of a linked-list holding int16 types
	static void printList16(adt::list<int16>* listToPrint)
	{
		adt::list<int16>::iterator Iter;	// create an iterator for the list

		Iter.set(listToPrint.begin());		// set the iterator to point to the start of the list

		while(!Iter.endOfList())			// loop until the end of the list is reached
		{
			io::cout.printNumber(Iter.get());	// print out the number we're currently pointing to
			io::cout.endl();					// new line

			Iter.next();						// point to the next element in the list
		}
	}

	// printListByte works the same way as above except it will print out a list holding 'bytes'
	static void printListByte(adt::list<byte>* listToPrint)
	{
		adt::list<byte>::iterator Iter;

		Iter.set(listToPrint.begin());

		while(!Iter.endOfList())
		{
			io::cout.printNumber((int16)Iter.get());
			io::cout.endl();

			Iter.next();
		}
	}

	static void main()
	{
		io::SCI.setup();				// setup the SCI port
		memory::heap dynamicMemory;		// create a new free store object
		dynamicMemory.setup(HEAP_MEMORY_START_ADDRESS, HEAP_MEMORY_SIZE);	// setup this free store using the defined values before

		adt::list<int16>* list1;		// create a pointer to a list holding int16 types
		list1 = adt::list<int16>.new(&dynamicMemory);	// set this list up

		adt::list<byte>* list2;			// create a pointer to a list holding byte types
		list2 = adt::list<byte>.new(&dynamicMemory);	// set this list up

		// now add some numbers to these lists:
		list1.add(5);
		list1.add(10);
		list1.add(15);
		list1.add(20);

		list2.add(1);
		list2.add(2);
		list2.add(3);
		list2.add(4);

		// and print them out:
		TestProgram3.printList16(list1);
		TestProgram3.printListByte(list2);
	}
}